#include <float.h>
#include <stdio.h>
#include <math.h>

#include <mkl.h>

#include "dot.h"
#include "timer.h"



float dot4(float *x, float *y )
{

   /*****************************************
    * Validation 
    *****************************************/
   float mkl_result = cblas_sdsdot (4, 0.0f, x, 1, y, 1);

   float rel_difference1 = fabs( (mkl_result - dot4_v1(x,y)) / mkl_result);
   float rel_difference2 = fabs( (mkl_result - dot4_v2(x,y)) / mkl_result);
   if(rel_difference1 < 1e-4)
      printf("Unit test 1 \033[32m SUCCESSFUL!\033[0m\n");
   else{
      printf("Unit test 1 \033[31m FAILED!\033[0m\n");
      exit(-1);
   }
   if(rel_difference2 < 1e-4)
      printf("Unit test 2 \033[32;1m SUCCESSFUL!\033[0m\n");
   else{
      printf("Unit test 2 \033[31m FAILED!\033[0m\n");
      exit(-1);
   }
   /****************************************/


   float result = 0.0f;

   double min_time = FLT_MAX;

   for(int j=0;j < 10000; ++j){

      unsigned long start, end;
      get_ticks(start);

      for(int i=0;i < 10000; ++i){
         result += dot4_v2(x,y);
      }

      get_ticks(end);
      double elapsed = ((double)(end-start))/FREQUENZY;
      if( elapsed < min_time )
         min_time = elapsed;
   }

   printf("Elapsed time: %g\n",min_time);

   return result;
}


float dot4_v1(float *x, float *y)
{
   float tmp[4];
//#pragma vector always
   for(int i=0;i < 4; ++i){
      tmp[i] = x[i] * y[i];
   }

   float result = 0.0f;
   for(int i=0;i < 4; ++i){
      result += tmp[i];
   }

   return result;
}

float dot4_v2(float *x, float *y)
{
   float result = 0.0f;
//#pragma vector always
   for(int i=0;i < 4; ++i){
      result += x[i] * y[i];
   }

   return result;
}



